#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for a cloud service."""

import urlparse
import libfal.falcmd as falcmd
import pal_info
from iobase.xmldata import (XmlConfNode, ANY_CHILD_NAME,)
from ionas.request import (Entry,)
from libfal.util import (get_proxy, NoProxy,)
from fal_info import (ReqTypes, Results,)

try:
    from landiskcloud.info.fletsazukeru import (FletsAzukeruConnectionInfo,)
except:
    pass

from pkgaddon.landisk_cloud_common_config import (CLOUD_CONF_FILE,)


CLOUD_NAME = "cloud"
ID_NAME = "id"
ID_MIN = 0
ANY_LIST_ITEM_XPATH = ANY_CHILD_NAME + "/" + ANY_CHILD_NAME
GET_ANY_ID_XPATH = ANY_LIST_ITEM_XPATH + "/id"
FIND_BY_ID_XPATH_TMPL = ANY_LIST_ITEM_XPATH + '/id[text()="%s"]/..'

FLETSAZUKERU_NAME = "fletsazukeru"
AREA_NAME = "area"
HTTPS_PROTOCOL = "https"

TO_LIST_TAG_NANE = {
        "dropbox": "dropboxlist",
        "amazons3": "amazons3list",
        "cloudn": "cloudnlist",
        "nifcloud": "nifcloudlist",
        "azure": "azurelist",
        "box": "boxlist",
        "onedrive": "onedrivelist",
        "onedrivebusiness": "onedrivebusinesslist",
        "googledrive": "googledrivelist",
        "fletsazukeru": "fletsazukerulist",
    }


class FalCloud(falcmd.FalCmd):
    """FAL Cloud command implementation class."""

    def __get_proxy(self, url):
        """Return proxy for "url"."""

        try:
            return get_proxy(url)
        except NoProxy:
            pass

        return ""

    def __get_proxy_for_flets_azukeru(self, host_name):
        """Return proxy for server corresponding a flets azukeru host_name."""

        target_url = urlparse.urlunparse((HTTPS_PROTOCOL, 
                                host_name, "", "", "", ""))

        return self.__get_proxy(target_url)

    def __load_config(self, file_name):
        """Load a 'file_name' file setting and return XML data ."""

        return self.send_entry_with_file(Entry(ReqTypes.LOAD),
                                        pal_info.PAL_CONFFILE_PATH,
                                        file_name).get()

    def __save_config(self, file_name, xml_data):
        """Save the "xml_data" XML data to a 'file_name' file."""

        entry = Entry(ReqTypes.SAVE)
        entry.set(xml_data)
        return self.send_entry_with_file(entry,
                                pal_info.PAL_CONFFILE_PATH, file_name)

    def __check_handler(self, entry):
        """Handle check request and set entry to result."""

        checked_node = entry.get().get_node(ANY_CHILD_NAME)
        cloud_conf = self.__load_config(CLOUD_CONF_FILE)
        try:
            target_id = checked_node.get_node(ID_NAME).get_text()
        except:
            self.exception("target id is not specified!!")
            entry.set_result(Results.CHECK_ERROR)
            return entry

        try:
            target_node = cloud_conf.get_node(FIND_BY_ID_XPATH_TMPL %
                            (target_id,))
        except:
            self.exception("target %d is not found" % (target_id,))
            entry.set_result(Results.CHECK_ERROR)
            return entry

        proxy_info = ""
        if target_node.get_node_name() == FLETSAZUKERU_NAME:
            connection_info = FletsAzukeruConnectionInfo(target_node)
            proxy_info = self.__get_proxy_for_flets_azukeru(
                                    connection_info.host_name)

        check_entry = Entry(ReqTypes.CHECK, CLOUD_NAME)
        check_entry.get().append_child(target_node)
        if proxy_info:
            check_entry.get().append_child(proxy_info)
        result_entry = self.send_entry(check_entry, pal_info.PAL_CLOUD_PATH)
        entry.set_result(result_entry.get_result())
        return entry

    def __add_handler(self, entry):
        """Handle add request and set entry to result."""

        added_node = entry.get().get_node(ANY_CHILD_NAME)
        try:
            # Force delete ID tag.
            added_node.delete_children(ID_NAME)
        except:
            # Ignore error.
            pass

        cloud_conf = self.__load_config(CLOUD_CONF_FILE)
        target_name = added_node.get_node_name()

        id_list = [int(id_str) for id_str in
                        cloud_conf.get_value_list(GET_ANY_ID_XPATH)]
        target_id = ID_MIN
        while True:
            if target_id in id_list:
                target_id += 1
                continue
            break

        targetlist = cloud_conf.get_node(TO_LIST_TAG_NANE[target_name])
        added_node.append_child(ID_NAME, str(target_id))
        targetlist.append_child(added_node)

        self.__save_config(CLOUD_CONF_FILE, cloud_conf)
        entry.set_result(Results.OK)
        return entry

    def __modify_handler(self, entry):
        """Handle modify request and set entry to result."""

        modified_node = entry.get().get_node(ANY_CHILD_NAME)
        cloud_conf = self.__load_config(CLOUD_CONF_FILE)
        target_node = cloud_conf.get_node(FIND_BY_ID_XPATH_TMPL %
                            (modified_node.get_node(ID_NAME).get_text(),))
        target_node.update(modified_node)

        self.__save_config(CLOUD_CONF_FILE, cloud_conf)
        entry.set_result(Results.OK)
        return entry

    def __delete_handler(self, entry):
        """Handle delete request and set entry to result."""

        deleted_node = entry.get().get_node(ANY_CHILD_NAME)
        cloud_conf = self.__load_config(CLOUD_CONF_FILE)

        for list_node in cloud_conf.get_node_list(ANY_CHILD_NAME):
            idx = 0
            for item in list_node.get_node_list(ANY_CHILD_NAME):
                if (item.get_value(ID_NAME) !=
                        deleted_node.get_value(ID_NAME)):

                    idx += 1
                    continue
                list_node.delete_child(ANY_CHILD_NAME, index=idx)

        self.__save_config(CLOUD_CONF_FILE, cloud_conf)
        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        save_and_load_handler = lambda entry: self.send_entry_with_file(
            entry, pal_info.PAL_CONFFILE_PATH, CLOUD_CONF_FILE)

        pass_handler = \
            lambda entry: self.send_entry(entry, pal_info.PAL_CLOUD_PATH)

        self.set_handler(ReqTypes.SAVE, save_and_load_handler)
        self.set_handler(ReqTypes.LOAD, save_and_load_handler)
        self.set_handler(ReqTypes.GET, pass_handler)
        self.set_handler(ReqTypes.CHECK, self.__check_handler)
        self.set_handler(ReqTypes.ADD, self.__add_handler)
        self.set_handler(ReqTypes.MODIFY, self.__modify_handler)
        self.set_handler(ReqTypes.DELETE, self.__delete_handler)

#
# Main
#

if __name__ == "__main__":
    FalCloud(option=falcmd.FalCmdOption()).handle_request()
